<?php if (!defined('PLX_ROOT')) exit;

##############################################################################################
#
# plnKikan. Plugin de gestion de choix de dates convenant à un maximum de participants.
# Auteur : Gari
# Date   : 25/02/2015
#
# Principe
# --------
# La page d'administration permet de créer un nouveau Kikan. Cela se traduit par :
# - une description
# - un titre
# - une liste de dates et d'horaires
#
# Côté site, une (fausse) page statique spécifique permet d'afficher la liste de tous les Kikan. 
# L'utilisateur n'a plus qu'à en choisir un, qui affiche le formulaire de choix.
# 
# Le Kikan
# --------
# Un Kikan contient, en sus des informations de paramétrage définis plus haut, l'ensemble
# des informations fournies par les participants.
# Un tableau avec un enregistrement par participant, chaque enregistrement ayant une valeur 
# numérique par date/horaire paramétré par le kikan. 
# 0 : ne peut pas venir
# 1 : peut-être
# 2 : peut venir
#
# Optionnel
# ---------
# Ce plugin peut être couplé à plnExtendedJurisdiction pour rendre privés certains Kikan, voire les 
# associer à des profils plus fins. Ce n'est cependant pas obligatoire : si plnExtendedJurisdiction
# n'est pas présent, il n'y aura simplement pas de privatisation possible. 
# 
##############################################################################################	

require_once("plnToolBox.class.php");

class plnKikan extends plxPlugin 
{
    public static $TimeStamp;
    public static $Date;
    public static $Time;
    public        $DataDir;
    public        $DataFile;
    public        $PluginDir;
    public        $aKikan;
    public        $kikanId = '';
    public static $isPrivacyPossible;
    public        $isAccesInterdit;     // true si le visiteur n'a pas les droits d'accès
    public static $isStaticPages;       // true si le module plnStaticPages est correctement chargé
    private static $nbInstances=0;

	##############################################################################################
	#
	# Le constructeur
	#
	##############################################################################################	
	public function __construct($default_lang) 
	{
        // On vérifie que le plugin n'est pas chargé plusieurs fois à tort
        self::$nbInstances++;
        if(self::$nbInstances !== 1)
            echo "Erreur : ".__CLASS__." est chargé ".self::$nbInstances." fois</br>";

		parent::__construct($default_lang);

		// Date du jour (à définir impérativement une seule fois pour éviter un bug au passage de minuit)
		self::$TimeStamp			  = date('Y-m-d H:i:s');
        list(self::$Date,self::$Time) = explode(" ",self::$TimeStamp);

		// Les répertoires
		$this->PluginDir = PLX_PLUGINS.__CLASS__."/";
        $this->DataDir   = PLX_ROOT.PLX_CONFIG_PATH."/plugins/";
        $this->DataFile  = $this->DataDir.__CLASS__."_data.xml";

        // Les droits d'accès à la page de configuration du plugin
        $this->setConfigProfil(PROFIL_ADMIN); 

        // On vérifie que le plugin technique facultatif plnExtendedJurisdiction est installé et chargé avant plnKikan
        self::$isPrivacyPossible = plnToolBox_v1::checkPlugin("plnExtendedJurisdiction",__CLASS__,"before");

        // On vérifie que le plugin technique obligatoire plnStaticPages est bien installé et chargé après plnKikan
        self::$isStaticPages     = plnToolBox_v1::checkPlugin("plnStaticPages",__CLASS__,"after");

        // Pour afficher les éventuelles erreurs
        $this->addHook('AdminTopBottom','AdminTopBottom');

        // Si StaticPages existe, on peut préparer tout le fonctionnement de plnKikan
        if(self::$isStaticPages)
        {
            // Les droits d'accès à la page d'administration du plugin
            switch($this->getParam("droitAcces"))
            {
                case 0  : $this->setAdminProfil(0);break; 
                case 1  : $this->setAdminProfil(0,1);break; 
                case 2  : $this->setAdminProfil(0,1,2);break; 
                case 3  : $this->setAdminProfil(0,1,2,3);break; 
                case 4  : $this->setAdminProfil(0,1,2,3,4);break; 
                default : $this->setAdminProfil(0);break; 
            }
            $this->setAdminMenu($this->getlang('Les Kikan'),'',$this->getlang('L_ADMIN_INFOBULLE'));

            // Est-ce qu'un utilisateur est en train de soumettre de nouveaux choix ?
            $this->addHook('plxMotorPreChauffageBegin','manageNewChoice');

            // Pour gérer les demandes d'affichage des kikans côté site. 
            $this->addHook('plxMotorPreChauffageBegin','plxMotorPreChauffageBegin');

            // Pour afficher le css spécifique au datepicker dans la partie admin
            $this->addHook('AdminTopEndHead','AdminTopEndHead');
        }
	}

    ##############################################################################################
    #
    # On affiche un message s'il y a eu des erreurs
    #
    ##############################################################################################
    public function AdminTopBottom() 
    {
        if(!self::$isStaticPages and (basename($_SERVER["PHP_SELF"]) == "parametres_plugins.php"))
            plnToolBox_v1::addMsg("Le plugin technique plnStaticPages doit être installé et chargé après le plugin plnKikan");
        plnToolBox_v1::displayMsg();
    }

    ##############################################################################################
    #
    # Pour le pikaday
    #
    ##############################################################################################
    public function AdminTopEndHead()
    {
        // On vérifie si on est sur la page admin, ce qui nous permettra d'ajouter les css associés au pikaday
        $plxAdmin = plxAdmin::getInstance();
        if($plxAdmin->path_url && preg_match("/plugin.php\?p=plnKikan&editkikan/",$plxAdmin->path_url))
            echo '<link rel="stylesheet" href="'.PLX_PLUGINS.'plnKikan/pikaday/pikaday.css">';
    }

    ##############################################################################################
    #
    # On récupère le nom du plugin ($this->plug est protected...)
    #
    ##############################################################################################  
    public function getName()   {return $this->plug["name"];}

    ##############################################################################################
    #
    # Est-ce que le visiteur a le droit d'accéder à la liste des kikans ?
    #
    ##############################################################################################  
    public function isGlobalAccessAllowed() 
    {
        if(!self::$isPrivacyPossible or $this->getParam("isPrivate") != 'true')
            return true;
        if(!plnExtendedJurisdiction::isConnected())
            return false;
        if($this->getParam("privateProfiles") != '' and !plnExtendedJurisdiction::isProfile($this->getParam("privateProfiles")))
            return false;
        return true;
    }

    ##############################################################################################
    #
    # Est-ce que le visiteur a le droit d'accéder à un kikan en particulier
    #
    ##############################################################################################  
    public function isSpecificAccessAllowed($kikanId) 
    {
        if(isset($this->aKikan[$kikanId]))
            $Kikan = $this->aKikan[$kikanId];
        else
            return true; // Le kikan n'existe pas, on peut donc bien donner l'accès...

        if(!self::$isPrivacyPossible or !$Kikan["isPrivate"])
            return true;
        if(!plnExtendedJurisdiction::isConnected())
            return false;
        if($Kikan["privateProfiles"] != '' and !plnExtendedJurisdiction::isProfile($Kikan["privateProfiles"]))
            return false;
        return true;
    }

    ##############################################################################################
    #
    # Le hook se lançant très tôt et permettant de récupérer les choix de l'utilisateur pour
    # les enregistrer
    #
    ##############################################################################################  
    public function manageNewChoice()
    {
        $this->plxMotor = plxMotor::getInstance();
        include_once(PLX_ROOT.'core/lib/class.plx.token.php');

        if(isset($_POST['kikanId']))
        {
            // On ne contrôle le token que si l'utilisateur fait une soumission de choix kikan
            plxToken::validateFormToken($_POST);

            $kikanId = $_POST['kikanId'];

            // On charge les kikan : on sait qu'on va en avoir besoin
            $this->loadKikan();

            // Le kikan n'existe pas ou n'est pas publié, on quitte.
            if(!isset($this->aKikan[$kikanId]) or $this->aKikan[$kikanId]['status'] != "publié")
                return;

            // Si l'utilisateur n'est pas autorisé à voir ce kikan, on quitte violemment.
            if(!$this->isSpecificAccessAllowed($kikanId))
                return;

            // Si le nom fourni est vide, on quitte
            if(empty($_POST['name']))
                return;

            $aTimestamp = array();
            foreach($this->aKikan[$kikanId]['dates'] as $date => $horaires)
            {
                foreach($horaires as $horaire)
                    $aTimestamp[] = $date."_".$horaire;
                if(empty($horaires))
                    $aTimestamp[] = $date;
            }

            // Pour le kikan en cours, on construit la liste des choix de l'utilisateur
            $Choix = array();
            foreach($aTimestamp as $timestamp)
                $Choix[$timestamp] = $_POST[$timestamp];

            $this->aKikan[$kikanId]['choix'][$_POST['name']] = $Choix;

            $this->saveKikan();

            unset($_POST);
        }
    }

    ##############################################################################################
    #
    # Méthode callback appelée par plnStaticPages quand on demande un plnKikan
    #
    ##############################################################################################  
    public function displayKikanFile()
    {
        // Quel fichier veut-on afficher (côté site) ?
        if($this->kikanId === '')
            include("list.plnKikan.php");
        else
            include("kikan.plnKikan.php");
    }

    ##############################################################################################
    #
    # Lorsque le moteur est en train de préchauffer, on intercepte éventuellement les
    # demandes d'affichage des kikans
    # Cela se fait en cliquant sur "Kikans" et envoie une commande de type index.php?plnKikan 
    # ou index.php?plnKikan<num>
    #
    ##############################################################################################  
    public function plxMotorPreChauffageBegin()
    {

        // On indique à plnStaticPages qu'on gère une "fausse" page statique 
        plnStaticPages::newStaticPage(__CLASS__,"plnKikan");
        // Veut-on un lien vers les kikans ? Si oui, dans quelles conditions ?
        if($this->getParam("inStaticList") == "true" 
        or ($this->getParam("inStaticList") == "private" and $this->isGlobalAccessAllowed()))
            plnStaticPages::setField("plnKikan","navLink",preg_replace("/ /","&nbsp;",$this->getlang("Les Kikan")));

        if(isset($_GET["plnKikan"]))
        {
            // On charge les kikan : on sait qu'on va en avoir besoin
            $this->loadKikan();

            // On récupère le hash du kikan demandé.
            $hash = plxUtils::strCheck($_GET["plnKikan"]);

            $this->kikanId = '';
            foreach($this->aKikan as $kikanId => $Kikan)
                if($Kikan['hash'] == $hash)
                {
                    $this->kikanId = $kikanId;
                    break;
                }

            if($this->kikanId !== '' and $this->aKikan[$this->kikanId]['status'] != 'publié')
                $this->kikanId  = '';

            // Quel fichier veut-on afficher ?
            $cible = $this->kikanId === '' ? 'list' : 'kikan';

            // Est-ce que l'accès à cette page est autorisé ? Si non, on demande à plnExtendedJurisdiction d'afficher
            // une page indiquant que l'accès est interdit.
            // Si on veut accéder à la liste, il faut vérifier qu'on a l'accès global.
            // Si on veut accéder à un kikan particulier, il faut vérifier qu'on a l'accès à ce kikan en particulier.
            if( ($cible == 'list'  and !$this->isGlobalAccessAllowed())
             or ($cible == 'kikan' and !$this->isSpecificAccessAllowed($this->kikanId)))
            {
                $this->isAccesInterdit = true;
                plnExtendedJurisdiction::AccesInterdit();
            }
            else
            {
                $this->isAccesInterdit = false;
                // Quel template ?
                $template = $this->getParam('template')=='' ? 'static.php' : $this->getParam('template');
                // Quel titres afficher ?
                if($this->kikanId === '')
                {
                    $pageTitle      = 'Kikans';
                    $staticTitle    = $this->getLang('L_KIKAN_LIST');

                }
                else
                {
                    $pageTitle      = 'Kikan '.$this->aKikan[$this->kikanId]['title'];
                    $staticTitle    = $this->getLang('L_KIKAN');

                }
                // On définit l'ensemble des titles et autres composantes accompagnant la page statique
                plnStaticPages::setField("plnKikan","staticTitle",$staticTitle);
                plnStaticPages::setField("plnKikan","pageTitle",$pageTitle);
                plnStaticPages::setField("plnKikan","template",$template);
                plnStaticPages::setField("plnKikan","method","displayKikanFile");

                // on renvoit un return true pour qu'il arrête le travail de MotorPrechauffageBegin, 
                // puisqu'on sait que ce sera un kikan
                echo "<?php return true ?>";
            }
        }
    }

    ##############################################################################################
    #
    # On charge le fichier des kikan, seulement si ce n'est déjà fait.
    #
    ##############################################################################################  
    public function loadKikan()
    {
        if(isset($this->aKikan) and is_array($this->aKikan))
            return;

        $this->aKikan = array();

        if(!is_readable($this->DataFile)) return;

        # Mise en place du parseur XML
        $xml = new SimpleXMLElement(implode('',file($this->DataFile)));

        # On parcourt les kikans
        $kikans = $xml->xpath('/document/kikan');
        foreach($kikans as $kikan)
        {
            $temp = array();
            $temp["title"]              = (string)$kikan->title;
            $temp["description"]        = (string)$kikan->description;
            $temp["nbDates"]            = (string)$kikan->nbDates;
            $temp["nbHoraires"]         = (string)$kikan->nbHoraires;
            $temp["typeKikan"]          = (string)$kikan->typeKikan;
            $temp["isPrivate"]          = $kikan->isPrivate == 1 ? true : false;
            $temp["privateProfiles"]    = (string)$kikan->privateProfiles;
            $temp["status"]             = (string)$kikan->status;
            $temp["dates"]              = array();
            $temp["choix"]              = array();
            $temp["hash"]               = (string)$kikan->hash;
            foreach($kikan->date as $date)
            {
                $temp["dates"][(string)$date['valeur']] = array();
                $j=0;
                foreach($date->horaire as $horaire)
                {
                    $temp["dates"][(string)$date['valeur']][$j] = (string)$horaire;
                    $j++;
                }
            }
            foreach($kikan->choix as $choix)
            {
                $temp["choix"][(string)$choix['name']] = array();
                foreach($choix->timestamp as $timestamp)
                    $temp["choix"][(string)$choix['name']][(string)$timestamp['valeur']] = (string)$timestamp['choix'];
            }
            $this->aKikan[(int)$kikan['id']] = $temp;
        }
    }

    ##############################################################################################
    #
    # On sauvegarde le fichier des kikan
    #
    ##############################################################################################  
    public function saveKikan()
    {
        # Début du fichier XML
        $xml = "<?xml version='1.0' encoding='".PLX_CHARSET."'?>\n";
        $xml .= "<document>\n";
        foreach($this->aKikan as $Id => $Kikan) 
        {
            $xml .= "  <kikan id=\"".$Id."\">\n";
            $xml .= "    <title><![CDATA[".$Kikan["title"]."]]></title>\n";
            $xml .= "    <description><![CDATA[".$Kikan["description"]."]]></description>\n";
            $xml .= "    <nbDates>".$Kikan["nbDates"]."</nbDates>\n";
            $xml .= "    <nbHoraires>".$Kikan["nbHoraires"]."</nbHoraires>\n";
            $xml .= "    <isPrivate>".$Kikan["isPrivate"]."</isPrivate>\n";
            $xml .= "    <privateProfiles>".$Kikan["privateProfiles"]."</privateProfiles>\n";
            $xml .= "    <status>".$Kikan["status"]."</status>\n";
            $xml .= "    <typeKikan>".$Kikan["typeKikan"]."</typeKikan>\n";
            $xml .= "    <hash>".$Kikan["hash"]."</hash>\n";
            ksort($Kikan["dates"]);
            foreach($Kikan["dates"] as $date => $horaires)
            {
 //               sort($horaires);
                $xml .= '    <date valeur="'.$date.'">'."\n";
                foreach($horaires as $horaire)
                $xml .= "      <horaire>".$horaire."</horaire>\n";
                $xml .= "    </date>\n";
            }
            foreach($Kikan['choix'] as $name => $timestamps)
            {
                $xml .= '    <choix name="'.$name.'">'."\n";
                foreach($timestamps as $timestamp => $choix)
                $xml .= '      <timestamp valeur="'.$timestamp.'" choix="'.$choix.'" />'."\n";
                $xml .= "    </choix>\n";
            }
            $xml .= "  </kikan>\n";
        }
        $xml .= "</document>\n";
        plxUtils::write($xml,$this->DataFile);

    }

    ##############################################################################################
    #
    # On créé un nouveau kikan
    #
    ##############################################################################################  
    public function newKikan($title,$description)
    {
        $this->loadKikan();

        $Kikan=array(   "title"         => $title,
                        "description"   => $description,
                        "private"       => $private,
                        "status"        => "brouillon",
                        "hash"          => sha1(plxUtils::charAleatoire(15)),
                        "dates"         => array());
        
        $this->aKikan[] = $Kikan;

        end($this->aKikan);
        $kikanId = key($this->aKikan);
        $this->aKikan[$kikanId]['hash'].=$kikanId; // On rajoute le numéro du kikan pour éviter une très improbable collision
        return($kikanId);
    }

    ##############################################################################################
    #
    # On détruit un kikan
    #
    ##############################################################################################  
    public function deleteKikan($kikanId)
    {
        $this->loadKikan();
        if(isset($this->aKikan[$kikanId]))
            unset($this->aKikan[$kikanId]);
    }
}

